@@ -84,16 +84,6 @@ namespace {
     lexer->advance(lexer, false);
   }
 
-  /**
-   * Gets the next codepoint in the string.
-   * 
-   * @param lexer The tree-sitter lexing control structure.
-   * @return The next codepoint in the string.
-   */
-  int32_t next_codepoint(const TSLexer* const lexer) {
-    return lexer->lookahead;
-  }
-
   /**
    * Checks whether the next codepoint is the one given.
    * 
@@ -105,7 +95,7 @@ namespace {
     const TSLexer* const lexer,
     int32_t const codepoint
   ) {
-    return codepoint == next_codepoint(lexer);
+    return codepoint == lexer->lookahead;
   }
 
   /**
@@ -115,7 +105,7 @@ namespace {
    * @return Whether there are any codepoints left in the string.
    */
   bool has_next(const TSLexer* const lexer) {
-    return !is_next_codepoint(lexer, 0);
+    return !lexer->eof(lexer);
   }
 
   /**
